"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.detectLDAPInjection = detectLDAPInjection;
const LDAP_INJECTION_PATTERNS = [
    /\(\|\|.*?\)/, // Logical OR operator in LDAP
    /\(\&\&.*?\)/, // Logical AND operator in LDAP
    /\b(&(.*?=.*?))\b/i, // AND query in LDAP
    /\b(\|(.*?=.*?))\b/i, // OR query in LDAP
    /\*\)/, // Closing parenthesis after an asterisk
    /\(.*?=.*?\)/i, // Parentheses containing an equals sign
    /\(\d+\)/, // Digits inside parentheses
    /\(\*\)/, // Asterisk inside parentheses
    /[\|\&\!]/, // Logical operators: |, &, !
    /\bobjectClass\b/i, // Common LDAP attribute
    /ldap:\/\/|ldaps:\/\//i, // LDAP URLs
    /\badmin\b/i, // Common LDAP user
    /\bOU=/i, // Organizational Unit
    /\bCN=/i, // Common Name
    /\bDC=/i, // Domain Component
    /\bUID=/i, // User ID
    /\buserPassword\b/i, // User password attribute
    /\bpass\b/i, // Password attribute
    /\w+\s*=\s*['"][^'"]*['"]/, // Attribute=value in quotes
    /base64\s*,\s*[A-Za-z0-9+/=]+/, // Base64 encoded data
    /\/\/\//, // Multiple forward slashes
    /(?:\b\w+\b\()|(?:\)\b\w+\b)/i, // Function-like patterns
];
/**
 * Detects if there is a potential LDAP injection in a given string.
 *
 * @param input - The string to analyze.
 * @returns `true` if a potential LDAP injection is detected, otherwise `false`.
 *
 * @remarks
 * This function uses a series of regular expressions to look for common LDAP injection patterns in the input string.
 * Patterns include logical operators, common LDAP attributes, and special characters used in LDAP queries.
 *
 * @example
 * ```typescript
 * const input = "(uid=admin)(&)";
 * const isLDAPInjection = detectLDAPInjection(input);
 * console.log(isLDAPInjection); // Output: true
 * ```
 *
 * @autor Andres Gutierrez Velez
 * @since 21/06/2024
 * @contact sr.willardkraft@gmail.com
 */
function detectLDAPInjection(input) {
    return LDAP_INJECTION_PATTERNS.some((pattern) => pattern.test(input));
}
//# sourceMappingURL=ldapInjectionDetector.js.map